searchNodes=[{"doc":"Allows you to paginate your Ecto results using cursors. Usage defmodule MyApp.Repo do use Ecto.Repo, otp_app: :my_app use Ouroboros end Options Ouroboros can take any options accepted by paginate/3. This is useful when you want to enforce some options globally across your project. Example defmodule MyApp.Repo do use Ecto.Repo, otp_app: :my_app use Ouroboros, limit: 10, limit_max: 100, end Note that these values can be still be overriden when paginate/3 is called. Use without macros If you wish to avoid use of macros or you wish to use a different name for the pagination function you can define your own function like so: defmodule MyApp.Repo do use Ecto.Repo, otp_app: :my_app def my_paginate_function(query, opts \\ [], repo_opts \\ []) do defaults = [limit: 10] # Default options of your choice here opts = Keyword.merge(defaults, opts) Ouroboros.paginate(query, opts, __MODULE__, repo_opts) end end","ref":"Ouroboros.html","title":"Ouroboros","type":"behaviour"},{"doc":"Generate a cursor for the supplied record, in the same manner as the before and after cursors generated by paginate/3. For the cursor to be compatible with paginate/3, fields must have the same value as the fields option passed to it. Example iex&gt; Ouroboros.cursor_for_record(%Ouroboros.Customer{id: 1}, [:id]) &quot;g2sAAQE&quot; iex&gt; Ouroboros.cursor_for_record(%Ouroboros.Customer{id: 1, name: &quot;Alice&quot;}, [id: :asc, name: :desc]) &quot;g2wAAAACYQFtAAAABUFsaWNlag&quot;","ref":"Ouroboros.html#cursor_for_record/3","title":"Ouroboros.cursor_for_record/3","type":"function"},{"doc":"Fetches all the results matching the query within the cursors. Options :after - fetch the records after this cursor; :before - fetch the records before this cursor; :fields - fields with sorting direction used to determine the cursor. In most cases, this should be the same fields as the ones used for sorting in the query. When you use named bindings in your query they can also be provided; :value_fun - function of arity 2 to lookup cursor values on returned records. Defaults to &amp;Ouroboros.value_fun_default/2; :limit - limits the number of records returned per page. Note that this number will be capped by :limit_max. Defaults to 50; :limit_max - sets a maximum cap for :limit. This option can be useful when :limit is set dynamically (e.g from a URL param set by a user) but you still want to enfore a maximum. Defaults to 100. Repo options This will be passed directly to Ecto.Repo.all/2, as such any option supported by this function can be used here. Simple example query = from(p in Post, order_by: [asc: p.inserted_at, asc: p.id], select: p) Repo.paginate(query, fields: [:inserted_at, :id], limit: 50) Example with using custom sort directions per field query = from(p in Post, order_by: [asc: p.inserted_at, desc: p.id], select: p) Repo.paginate(query, fields: [inserted_at: :asc, id: :desc], limit: 50) Example with sorting on columns in joined tables from p in Post, as: :posts, join: a in assoc(p, :author), as: :author, preload: [author: a], select: p, order_by: [asc: a.name, asc: p.id] Repo.paginate(query, fields: [{{:author, :name}, :asc}, id: :asc], limit: 50) When sorting on columns in joined tables it is necessary to use named bindings. In this case we name it author. In the fields we refer to this named binding and its column name. To build the cursor Ouroboros uses the returned Ecto.Schema. When using a joined column the returned Ecto.Schema won&#39;t have the value of the joined column unless we preload it. E.g. in this case the cursor will be build up from post.id and post.author.name. This presupposes that the named of the binding is the same as the name of the relationship on the original struct. One level deep joins are supported out of the box but if we join on a second level, e.g. post.author.company.name a custom function can be supplied to handle the cursor value retrieval. This also applies when the named binding does not map to the name of the relationship. Example from p in Post, as: :posts, join: a in assoc(p, :author), as: :author, join: c in assoc(a, :company), as: :company, preload: [author: a], select: p, order_by: [ {:asc, a.name}, {:asc, p.id} ] Repo.paginate(query, fields: [{{:company, :name}, :asc}, id: :asc], value_fun: fn post, {{:company, name}, _} -&gt; {:string, post.author.company.name} post, field -&gt; Ouroboros.value_fun_default(post, field) end, limit: 50)","ref":"Ouroboros.html#c:paginate/3","title":"Ouroboros.paginate/3","type":"callback"},{"doc":"Default function used to get the value of a cursor field from the supplied map. This function can be overriden in the Ouroboros.Config using the value_fun key. When using named bindings to sort on joined columns it will attempt to get the value of joined column by using the named binding as the name of the relationship on the original Ecto.Schema. Example iex&gt; Ouroboros.type_fun_default(%Ouroboros.Customer{id: 1}, :id) :id iex&gt; Ouroboros.type_fun_default(%Ouroboros.Customer{}, {:address, :city}) :string iex&gt; Ouroboros.type_fun_default(%Ouroboros.Customer{id: 1, address: %Ouroboros.Address{city: &quot;London&quot;}}, {:address, :city}) :string iex&gt; Ouroboros.type_fun_default(Ouroboros.Customer, :inserted_at) :naive_datetime","ref":"Ouroboros.html#type_fun_default/2","title":"Ouroboros.type_fun_default/2","type":"function"},{"doc":"Default function used to get the value of a cursor field from the supplied map. This function can be overriden in the Ouroboros.Config using the value_fun key. When using named bindings to sort on joined columns it will attempt to get the value of joined column by using the named binding as the name of the relationship on the original Ecto.Schema. Example iex&gt; Ouroboros.value_fun_default(%Ouroboros.Customer{id: 1}, :id) {:id, 1} iex&gt; Ouroboros.value_fun_default(%Ouroboros.Customer{id: 1, address: %Ouroboros.Address{city: &quot;London&quot;}}, {:address, :city}) {:string, &quot;London&quot;}","ref":"Ouroboros.html#value_fun_default/2","title":"Ouroboros.value_fun_default/2","type":"function"},{"doc":"","ref":"Ouroboros.Address.html","title":"Ouroboros.Address","type":"module"},{"doc":"","ref":"Ouroboros.Case.html","title":"Ouroboros.Case","type":"module"},{"doc":"","ref":"Ouroboros.Customer.html","title":"Ouroboros.Customer","type":"module"},{"doc":"","ref":"Ouroboros.Customer.html#active/1","title":"Ouroboros.Customer.active/1","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html","title":"Ouroboros.Factory","type":"module"},{"doc":"","ref":"Ouroboros.Factory.html#address_factory/0","title":"Ouroboros.Factory.address_factory/0","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#build/2","title":"Ouroboros.Factory.build/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#build_list/3","title":"Ouroboros.Factory.build_list/3","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#build_pair/2","title":"Ouroboros.Factory.build_pair/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#create/1","title":"Ouroboros.Factory.create/1","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#create/2","title":"Ouroboros.Factory.create/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#create_list/3","title":"Ouroboros.Factory.create_list/3","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#create_pair/2","title":"Ouroboros.Factory.create_pair/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#customer_factory/0","title":"Ouroboros.Factory.customer_factory/0","type":"function"},{"doc":"Raises a helpful error if no factory is defined.","ref":"Ouroboros.Factory.html#factory/1","title":"Ouroboros.Factory.factory/1","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#insert/1","title":"Ouroboros.Factory.insert/1","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#insert/2","title":"Ouroboros.Factory.insert/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#insert_list/3","title":"Ouroboros.Factory.insert_list/3","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#insert_pair/2","title":"Ouroboros.Factory.insert_pair/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#params_for/2","title":"Ouroboros.Factory.params_for/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#params_with_assocs/2","title":"Ouroboros.Factory.params_with_assocs/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#payment_factory/0","title":"Ouroboros.Factory.payment_factory/0","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#string_params_for/2","title":"Ouroboros.Factory.string_params_for/2","type":"function"},{"doc":"","ref":"Ouroboros.Factory.html#string_params_with_assocs/2","title":"Ouroboros.Factory.string_params_with_assocs/2","type":"function"},{"doc":"","ref":"Ouroboros.Migration.html","title":"Ouroboros.Migration","type":"module"},{"doc":"","ref":"Ouroboros.Migration.html#change/0","title":"Ouroboros.Migration.change/0","type":"function"},{"doc":"Defines a page. Fields entries - a list entries contained in this page. metadata - metadata attached to this page.","ref":"Ouroboros.Page.html","title":"Ouroboros.Page","type":"module"},{"doc":"","ref":"Ouroboros.Page.html#t:t/0","title":"Ouroboros.Page.t/0","type":"type"},{"doc":"Defines page metadata. Fields after - cursor representing the last row of the current page; before - cursor representing the first row of the current page; limit - the maximum number of entries that can be contained in this page.","ref":"Ouroboros.Page.Metadata.html","title":"Ouroboros.Page.Metadata","type":"module"},{"doc":"","ref":"Ouroboros.Page.Metadata.html#t:t/0","title":"Ouroboros.Page.Metadata.t/0","type":"type"},{"doc":"","ref":"Ouroboros.Payment.html","title":"Ouroboros.Payment","type":"module"},{"doc":"","ref":"Ouroboros.Payment.html#failed/1","title":"Ouroboros.Payment.failed/1","type":"function"},{"doc":"","ref":"Ouroboros.Payment.html#successful/1","title":"Ouroboros.Payment.successful/1","type":"function"},{"doc":"","ref":"Ouroboros.Repo.html","title":"Ouroboros.Repo","type":"module"},{"doc":"Callback implementation for Ecto.Repo.aggregate/3.","ref":"Ouroboros.Repo.html#aggregate/3","title":"Ouroboros.Repo.aggregate/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.aggregate/4.","ref":"Ouroboros.Repo.html#aggregate/4","title":"Ouroboros.Repo.aggregate/4","type":"function"},{"doc":"Callback implementation for Ecto.Repo.all/2.","ref":"Ouroboros.Repo.html#all/2","title":"Ouroboros.Repo.all/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.checkout/2.","ref":"Ouroboros.Repo.html#checkout/2","title":"Ouroboros.Repo.checkout/2","type":"function"},{"doc":"","ref":"Ouroboros.Repo.html#child_spec/1","title":"Ouroboros.Repo.child_spec/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.config/0.","ref":"Ouroboros.Repo.html#config/0","title":"Ouroboros.Repo.config/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.delete/2.","ref":"Ouroboros.Repo.html#delete/2","title":"Ouroboros.Repo.delete/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.delete!/2.","ref":"Ouroboros.Repo.html#delete!/2","title":"Ouroboros.Repo.delete!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.delete_all/2.","ref":"Ouroboros.Repo.html#delete_all/2","title":"Ouroboros.Repo.delete_all/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.exists?/2.","ref":"Ouroboros.Repo.html#exists?/2","title":"Ouroboros.Repo.exists?/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get/3.","ref":"Ouroboros.Repo.html#get/3","title":"Ouroboros.Repo.get/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get!/3.","ref":"Ouroboros.Repo.html#get!/3","title":"Ouroboros.Repo.get!/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get_by/3.","ref":"Ouroboros.Repo.html#get_by/3","title":"Ouroboros.Repo.get_by/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get_by!/3.","ref":"Ouroboros.Repo.html#get_by!/3","title":"Ouroboros.Repo.get_by!/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get_dynamic_repo/0.","ref":"Ouroboros.Repo.html#get_dynamic_repo/0","title":"Ouroboros.Repo.get_dynamic_repo/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.in_transaction?/0.","ref":"Ouroboros.Repo.html#in_transaction?/0","title":"Ouroboros.Repo.in_transaction?/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.init/2.","ref":"Ouroboros.Repo.html#init/2","title":"Ouroboros.Repo.init/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert/2.","ref":"Ouroboros.Repo.html#insert/2","title":"Ouroboros.Repo.insert/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert!/2.","ref":"Ouroboros.Repo.html#insert!/2","title":"Ouroboros.Repo.insert!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert_all/3.","ref":"Ouroboros.Repo.html#insert_all/3","title":"Ouroboros.Repo.insert_all/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert_or_update/2.","ref":"Ouroboros.Repo.html#insert_or_update/2","title":"Ouroboros.Repo.insert_or_update/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert_or_update!/2.","ref":"Ouroboros.Repo.html#insert_or_update!/2","title":"Ouroboros.Repo.insert_or_update!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.load/2.","ref":"Ouroboros.Repo.html#load/2","title":"Ouroboros.Repo.load/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.one/2.","ref":"Ouroboros.Repo.html#one/2","title":"Ouroboros.Repo.one/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.one!/2.","ref":"Ouroboros.Repo.html#one!/2","title":"Ouroboros.Repo.one!/2","type":"function"},{"doc":"","ref":"Ouroboros.Repo.html#paginate/3","title":"Ouroboros.Repo.paginate/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.preload/3.","ref":"Ouroboros.Repo.html#preload/3","title":"Ouroboros.Repo.preload/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.prepare_query/3.","ref":"Ouroboros.Repo.html#prepare_query/3","title":"Ouroboros.Repo.prepare_query/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.put_dynamic_repo/1.","ref":"Ouroboros.Repo.html#put_dynamic_repo/1","title":"Ouroboros.Repo.put_dynamic_repo/1","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes the given query. See Ecto.Adapters.SQL.query/4 for more information.","ref":"Ouroboros.Repo.html#query/3","title":"Ouroboros.Repo.query/3","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes the given query. See Ecto.Adapters.SQL.query!/4 for more information.","ref":"Ouroboros.Repo.html#query!/3","title":"Ouroboros.Repo.query!/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.rollback/1.","ref":"Ouroboros.Repo.html#rollback/1","title":"Ouroboros.Repo.rollback/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.start_link/1.","ref":"Ouroboros.Repo.html#start_link/1","title":"Ouroboros.Repo.start_link/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.stop/1.","ref":"Ouroboros.Repo.html#stop/1","title":"Ouroboros.Repo.stop/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.stream/2.","ref":"Ouroboros.Repo.html#stream/2","title":"Ouroboros.Repo.stream/2","type":"function"},{"doc":"A convenience function for SQL-based repositories that translates the given query to SQL. See Ecto.Adapters.SQL.to_sql/3 for more information.","ref":"Ouroboros.Repo.html#to_sql/2","title":"Ouroboros.Repo.to_sql/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.transaction/2.","ref":"Ouroboros.Repo.html#transaction/2","title":"Ouroboros.Repo.transaction/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.update/2.","ref":"Ouroboros.Repo.html#update/2","title":"Ouroboros.Repo.update/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.update!/2.","ref":"Ouroboros.Repo.html#update!/2","title":"Ouroboros.Repo.update!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.update_all/3.","ref":"Ouroboros.Repo.html#update_all/3","title":"Ouroboros.Repo.update_all/3","type":"function"}]